@@ -14,10 +14,13 @@
 
 #include "nsAtom.h"
 #include "nsIContent.h"
-#include "mozilla/dom/Element.h"
 
 class nsINode;
 
+namespace mozilla::dom {
+class Element;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Value constants
 
@@ -71,7 +74,12 @@ enum EActionRule {
 /**
  * Used to define if role exposes default value of aria-live attribute.
  */
-enum ELiveAttrRule { eNoLiveAttr, eOffLiveAttr, ePoliteLiveAttr };
+enum ELiveAttrRule {
+  eNoLiveAttr,
+  eOffLiveAttr,
+  ePoliteLiveAttr,
+  eAssertiveLiveAttr
+};
 
 ////////////////////////////////////////////////////////////////////////////////
 // Role constants
@@ -129,7 +137,7 @@ struct nsRoleMapEntry {
   /**
    * Return true if matches to the given ARIA role.
    */
-  bool Is(nsAtom* aARIARole) const { return *roleAtom == aARIARole; }
+  bool Is(nsAtom* aARIARole) const { return roleAtom == aARIARole; }
 
   /**
    * Return true if ARIA role has the given accessible type.
@@ -142,11 +150,11 @@ struct nsRoleMapEntry {
    * Return ARIA role.
    */
   const nsDependentAtomString ARIARoleString() const {
-    return nsDependentAtomString(*roleAtom);
+    return nsDependentAtomString(roleAtom);
   }
 
   // ARIA role: string representation such as "button"
-  nsStaticAtom** roleAtom;
+  nsStaticAtom* const roleAtom;
 
   // Role mapping rule: maps to enum Role
   mozilla::a11y::role role;
@@ -164,7 +172,7 @@ struct nsRoleMapEntry {
   // these object attributes if ARIA 'live' attribute is missed.
   ELiveAttrRule liveAttRule;
 
-  // Accessible types this role belongs to.
+  // LocalAccessible types this role belongs to.
   uint32_t accTypes;
 
   // Automatic state mapping rule: always include in states
@@ -254,7 +262,7 @@ uint8_t GetIndexFromRoleMap(const nsRoleMapEntry* aRoleMap);
  * Return accessible state from ARIA universal states applied to the given
  * element.
  */
-uint64_t UniversalStatesFor(mozilla::dom::Element* aElement);
+uint64_t UniversalStatesFor(dom::Element* aElement);
 
 /**
  * Get the ARIA attribute characteristics for a given ARIA attribute.
@@ -276,13 +284,15 @@ bool HasDefinedARIAHidden(nsIContent* aContent);
  */
 class AttrIterator {
  public:
-  explicit AttrIterator(nsIContent* aContent)
-      : mElement(aContent->IsElement() ? aContent->AsElement() : nullptr),
-        mAttrIdx(0) {
-    mAttrCount = mElement ? mElement->GetAttrCount() : 0;
-  }
+  explicit AttrIterator(nsIContent* aContent);
 
-  bool Next(nsAString& aAttrName, nsAString& aAttrValue);
+  bool Next();
+
+  void AttrName(nsAString& aAttrName) const;
+
+  nsAtom* AttrName() const;
+
+  void AttrValue(nsAString& aAttrValue) const;
 
  private:
   AttrIterator() = delete;
@@ -292,6 +302,7 @@ class AttrIterator {
   dom::Element* mElement;
   uint32_t mAttrIdx;
   uint32_t mAttrCount;
+  RefPtr<nsAtom> mAttrAtom;
 };
 
 }  // namespace aria
