@@ -17,10 +17,18 @@ var {
   CalDavOutboxRequest,
   CalDavFreeBusyRequest,
 } = ChromeUtils.import("resource:///modules/caldav/CalDavRequest.jsm");
+var { CalDavWebDavSyncHandler } = ChromeUtils.import(
+  "resource:///modules/caldav/CalDavRequestHandlers.jsm"
+);
 
 var { CalDavSession } = ChromeUtils.import("resource:///modules/caldav/CalDavSession.jsm");
 var { CalDavXmlns } = ChromeUtils.import("resource:///modules/caldav/CalDavUtils.jsm");
 var { Preferences } = ChromeUtils.import("resource://gre/modules/Preferences.jsm");
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalEvent: "resource:///modules/CalEvent.jsm",
+});
 
 class LowerMap extends Map {
   get(key) {
@@ -54,10 +62,8 @@ var MockConflictPrompt = {
 };
 
 class MockAlertsService {
-  QueryInterface(aIID) {
-    return cal.generateClassQI(this, aIID, [Ci.nsIAlertsService]);
-  }
-  showAlertNotification() {}
+  QueryInterface = ChromeUtils.generateQI(["nsIAlertsService"]);
+  showAlert() {}
 }
 
 function replaceAlertsService() {
@@ -70,7 +76,27 @@ function replaceAlertsService() {
   });
 }
 
-var gMockCalendar = { name: "xpcshell" };
+var gMockCalendar = {
+  name: "xpcshell",
+  makeUri(insert, base) {
+    return base;
+  },
+  verboseLogging() {
+    return true;
+  },
+  ensureEncodedPath(x) {
+    return x;
+  },
+  ensureDecodedPath(x) {
+    return x;
+  },
+  startBatch() {},
+  endBatch() {},
+  addTargetCalendarItem() {},
+  finalizeUpdatedItems() {},
+  mHrefIndex: [],
+};
+gMockCalendar.superCalendar = gMockCalendar;
 
 class CalDavServer {
   constructor(calendarId) {
@@ -385,6 +411,7 @@ class CalDavServer {
       response.setHeader("Content-Type", "application/xml");
       let bodydom = cal.xml.parseString(body);
       let report = bodydom.documentElement.localName;
+      let eventName = String.fromCharCode(...new TextEncoder().encode("イベント"));
       if (report == "sync-collection") {
         response.write(dedent`
           <?xml version="1.0" encoding="utf-8" ?>
@@ -395,6 +422,7 @@ class CalDavServer {
                 <D:prop>
                   <D:getcontenttype>text/calendar; charset=utf-8; component=VEVENT</D:getcontenttype>
                   <D:getetag>"2decee6ffb701583398996bfbdacb8eec53edf94"</D:getetag>
+                  <D:displayname>${eventName}</D:displayname>
                 </D:prop>
                 <D:status>HTTP/1.1 200 OK</D:status>
               </D:propstat>
@@ -402,9 +430,11 @@ class CalDavServer {
           </D:multistatus>
         `);
       } else if (report == "calendar-multiget") {
-        let event = cal.createEvent();
+        let event = new CalEvent();
+        event.title = "会議";
         event.startDate = cal.dtz.now();
         event.endDate = cal.dtz.now();
+        let icalString = String.fromCharCode(...new TextEncoder().encode(event.icalString));
         response.write(dedent`
           <?xml version="1.0" encoding="utf-8"?>
           <D:multistatus ${CalDavXmlns("D", "C")}>
@@ -413,7 +443,7 @@ class CalDavServer {
               <D:propstat>
                 <D:prop>
                   <D:getetag>"2decee6ffb701583398996bfbdacb8eec53edf94"</D:getetag>
-                  <C:calendar-data>${event.icalString}</C:calendar-data>
+                  <C:calendar-data>${icalString}</C:calendar-data>
                 </D:prop>
               </D:propstat>
             </D:response>
@@ -642,7 +672,7 @@ add_task(async function test_item_request() {
     gServer.session,
     gMockCalendar,
     uri,
-    cal.createEvent(icalString),
+    new CalEvent(icalString),
     "*"
   );
   let response = await request.commit();
@@ -664,7 +694,7 @@ add_task(async function test_item_request() {
     gServer.session,
     gMockCalendar,
     uri,
-    cal.createEvent(icalString),
+    new CalEvent(icalString),
     "123123"
   );
   response = await request.commit();
@@ -682,7 +712,7 @@ add_task(async function test_item_request() {
   // Now the same with 200 OK and no etag
   gServer.reset();
   uri = gServer.uri("/requests/item/200");
-  request = new CalDavItemRequest(gServer.session, gMockCalendar, uri, cal.createEvent(icalString));
+  request = new CalDavItemRequest(gServer.session, gMockCalendar, uri, new CalEvent(icalString));
   response = await request.commit();
 
   equal(response.status, 200);
@@ -829,7 +859,7 @@ add_task(async function test_outbox_request() {
     "xpcshell@example.com",
     ["recipient1@example.com", "recipient2@example.com"],
     "REPLY",
-    cal.createEvent(icalString)
+    new CalEvent(icalString)
   );
   let response = await request.commit();
 
@@ -922,4 +952,17 @@ add_task(async function test_caldav_client() {
 
   let items = await pclient.getAllItems();
   equal(items.length, 1);
+  equal(items[0].title, "会議");
+});
+
+/**
+ * Test non-ASCII text in the XML response is parsed correctly in CalDavWebDavSyncHandler.
+ */
+add_task(async function test_caldav_sync() {
+  gServer.reset();
+  let uri = gServer.uri("/calendars/xpcshell/events/");
+  gMockCalendar.session = gServer.session;
+  let webDavSync = new CalDavWebDavSyncHandler(gMockCalendar, uri);
+  await webDavSync.doWebDAVSync();
+  ok(webDavSync.logXML.includes("イベント"), "Non-ASCII text should be parsed correctly");
 });
