@@ -11,7 +11,7 @@ const worker = {
   state: PropTypes.number.isRequired,
   stateText: PropTypes.string.isRequired,
   url: PropTypes.string.isRequired,
-  workerTargetFront: PropTypes.object,
+  workerDescriptorFront: PropTypes.object,
   registrationFront: PropTypes.object,
 };
 
