@@ -7,13 +7,13 @@
 #include <string.h>
 
 #if defined(ANDROID)
-#include <fcntl.h>
-#include <linux/ashmem.h>
-#include <sys/ioctl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
+#  include <fcntl.h>
+#  include <linux/ashmem.h>
+#  include <sys/ioctl.h>
+#  include <sys/stat.h>
+#  include <sys/types.h>
 #elif defined(XP_DARWIN)
-#include <mach/mach.h>
+#  include <mach/mach.h>
 #endif
 
 using namespace mozilla;
@@ -22,13 +22,15 @@ TEST(VolatileBufferTest, HeapVolatileBuffersWork)
 {
   RefPtr<VolatileBuffer> heapbuf = new VolatileBuffer();
 
-  ASSERT_TRUE(heapbuf) << "Failed to create VolatileBuffer";
-  ASSERT_TRUE(heapbuf->Init(512)) << "Failed to initialize VolatileBuffer";
+  ASSERT_TRUE(heapbuf)
+  << "Failed to create VolatileBuffer";
+  ASSERT_TRUE(heapbuf->Init(512))
+  << "Failed to initialize VolatileBuffer";
 
   VolatileBufferPtr<char> ptr(heapbuf);
 
   EXPECT_FALSE(ptr.WasBufferPurged())
-    << "Buffer should not be purged immediately after initialization";
+      << "Buffer should not be purged immediately after initialization";
   EXPECT_TRUE(ptr) << "Couldn't get pointer from VolatileBufferPtr";
 }
 
@@ -36,8 +38,10 @@ TEST(VolatileBufferTest, RealVolatileBuffersWork)
 {
   RefPtr<VolatileBuffer> buf = new VolatileBuffer();
 
-  ASSERT_TRUE(buf) << "Failed to create VolatileBuffer";
-  ASSERT_TRUE(buf->Init(16384)) << "Failed to initialize VolatileBuffer";
+  ASSERT_TRUE(buf)
+  << "Failed to create VolatileBuffer";
+  ASSERT_TRUE(buf->Init(16384))
+  << "Failed to initialize VolatileBuffer";
 
   const char teststr[] = "foobar";
 
@@ -45,15 +49,16 @@ TEST(VolatileBufferTest, RealVolatileBuffersWork)
     VolatileBufferPtr<char> ptr(buf);
 
     EXPECT_FALSE(ptr.WasBufferPurged())
-      << "Buffer should not be purged immediately after initialization";
+        << "Buffer should not be purged immediately after initialization";
     EXPECT_TRUE(ptr) << "Couldn't get pointer from VolatileBufferPtr";
 
     {
       VolatileBufferPtr<char> ptr2(buf);
 
       EXPECT_FALSE(ptr.WasBufferPurged())
-        << "Failed to lock buffer again while currently locked";
-      ASSERT_TRUE(ptr2) << "Didn't get a pointer on the second lock";
+          << "Failed to lock buffer again while currently locked";
+      ASSERT_TRUE(ptr2)
+      << "Didn't get a pointer on the second lock";
 
       strcpy(ptr2, teststr);
     }
@@ -63,7 +68,7 @@ TEST(VolatileBufferTest, RealVolatileBuffersWork)
     VolatileBufferPtr<char> ptr(buf);
 
     EXPECT_FALSE(ptr.WasBufferPurged())
-      << "Buffer was immediately purged after unlock";
+        << "Buffer was immediately purged after unlock";
     EXPECT_STREQ(ptr, teststr) << "Buffer failed to retain data after unlock";
   }
 
@@ -77,13 +82,13 @@ TEST(VolatileBufferTest, RealVolatileBuffersWork)
 #endif
 
   EXPECT_GT(buf->NonHeapSizeOfExcludingThis(), 0ul)
-    << "Buffer should not be allocated on heap";
+      << "Buffer should not be allocated on heap";
 
   {
     VolatileBufferPtr<char> ptr(buf);
 
     EXPECT_TRUE(ptr.WasBufferPurged())
-      << "Buffer should not be unpurged after forced purge";
+        << "Buffer should not be unpurged after forced purge";
     EXPECT_STRNE(ptr, teststr) << "Purge did not actually purge data";
   }
 
