@@ -12,12 +12,16 @@
 #include "nsNetCID.h"
 #include "nsQueryObject.h"
 #include "mozilla/ClearOnShutdown.h"
+#include "mozilla/StaticPrefs_network.h"
 #include "mozilla/StaticPtr.h"
 #include "mozilla/SyncRunnable.h"
 #include "mozilla/net/NeckoChild.h"
 #include "mozilla/net/DNSListenerProxy.h"
 #include "mozilla/net/TRRServiceParent.h"
+#include "nsHostResolver.h"
 #include "nsServiceManagerUtils.h"
+#include "prsystem.h"
+#include "DNSResolverInfo.h"
 
 namespace mozilla {
 namespace net {
@@ -36,27 +40,12 @@ already_AddRefed<ChildDNSService> ChildDNSService::GetSingleton() {
                 XRE_IsContentProcess() || XRE_IsSocketProcess());
 
   if (!gChildDNSService) {
-    auto initTask = []() {
-      gChildDNSService = new ChildDNSService();
-      ClearOnShutdown(&gChildDNSService);
-    };
-
-    // For normal cases, DNS service should be initialized in nsHttpHandler. For
-    // some xpcshell tests, nsHttpHandler is not used at all, so the best we use
-    // SyncRunnable to make sure DNS service is initialized on main thread.
-    if (!NS_IsMainThread()) {
-      // Forward to the main thread synchronously.
-      RefPtr<nsIThread> mainThread = do_GetMainThread();
-      if (!mainThread) {
-        return nullptr;
-      }
-
-      SyncRunnable::DispatchToThread(
-          mainThread, new SyncRunnable(NS_NewRunnableFunction(
-                          "ChildDNSService::GetSingleton", initTask)));
-    } else {
-      initTask();
+    if (NS_WARN_IF(!NS_IsMainThread())) {
+      return nullptr;
     }
+    gChildDNSService = new ChildDNSService();
+    gChildDNSService->Init();
+    ClearOnShutdown(&gChildDNSService);
   }
 
   return do_AddRef(gChildDNSService);
@@ -64,10 +53,7 @@ already_AddRefed<ChildDNSService> ChildDNSService::GetSingleton() {
 
 NS_IMPL_ISUPPORTS(ChildDNSService, nsIDNSService, nsPIDNSService, nsIObserver)
 
-ChildDNSService::ChildDNSService()
-    : mFirstTime(true),
-      mDisablePrefetch(false),
-      mPendingRequestsLock("DNSPendingRequestsLock") {
+ChildDNSService::ChildDNSService() {
   MOZ_ASSERT_IF(nsIOService::UseSocketProcess(),
                 XRE_IsContentProcess() || XRE_IsParentProcess());
   MOZ_ASSERT_IF(!nsIOService::UseSocketProcess(),
@@ -96,9 +82,10 @@ void ChildDNSService::GetDNSRecordHashKey(
 }
 
 nsresult ChildDNSService::AsyncResolveInternal(
-    const nsACString& hostname, const nsACString& aTrrServer, uint16_t type,
-    uint32_t flags, nsIDNSListener* listener, nsIEventTarget* target_,
-    const OriginAttributes& aOriginAttributes, nsICancelable** result) {
+    const nsACString& hostname, uint16_t type, uint32_t flags,
+    nsIDNSResolverInfo* aResolver, nsIDNSListener* listener,
+    nsIEventTarget* target_, const OriginAttributes& aOriginAttributes,
+    nsICancelable** result) {
   if (XRE_IsContentProcess()) {
     NS_ENSURE_TRUE(gNeckoChild != nullptr, NS_ERROR_FAILURE);
   }
@@ -106,6 +93,11 @@ nsresult ChildDNSService::AsyncResolveInternal(
   bool resolveDNSInSocketProcess = false;
   if (XRE_IsParentProcess() && nsIOService::UseSocketProcess()) {
     resolveDNSInSocketProcess = true;
+    if (type != nsIDNSService::RESOLVE_TYPE_DEFAULT &&
+        (mTRRServiceParent->Mode() != nsIDNSService::MODE_TRRFIRST &&
+         mTRRServiceParent->Mode() != nsIDNSService::MODE_TRRONLY)) {
+      return NS_ERROR_UNKNOWN_HOST;
+    }
   }
 
   if (mDisablePrefetch && (flags & RESOLVE_SPECULATE)) {
@@ -127,8 +119,9 @@ nsresult ChildDNSService::AsyncResolveInternal(
     listener = new DNSListenerProxy(listener, target);
   }
 
-  RefPtr<DNSRequestSender> sender = new DNSRequestSender(
-      hostname, aTrrServer, type, aOriginAttributes, flags, listener, target);
+  RefPtr<DNSRequestSender> sender =
+      new DNSRequestSender(hostname, DNSResolverInfo::URL(aResolver), type,
+                           aOriginAttributes, flags, listener, target);
   RefPtr<DNSRequestActor> dnsReq;
   if (resolveDNSInSocketProcess) {
     dnsReq = new DNSRequestParent(sender);
@@ -139,18 +132,9 @@ nsresult ChildDNSService::AsyncResolveInternal(
   {
     MutexAutoLock lock(mPendingRequestsLock);
     nsCString key;
-    GetDNSRecordHashKey(hostname, aTrrServer, type, aOriginAttributes, flags,
-                        originalListenerAddr, key);
-    auto entry = mPendingRequests.LookupForAdd(key);
-    if (entry) {
-      entry.Data()->AppendElement(sender);
-    } else {
-      entry.OrInsert([&]() {
-        auto* hashEntry = new nsTArray<RefPtr<DNSRequestSender>>();
-        hashEntry->AppendElement(sender);
-        return hashEntry;
-      });
-    }
+    GetDNSRecordHashKey(hostname, DNSResolverInfo::URL(aResolver), type,
+                        aOriginAttributes, flags, originalListenerAddr, key);
+    mPendingRequests.GetOrInsertNew(key)->AppendElement(sender);
   }
 
   sender->StartRequest();
@@ -160,8 +144,8 @@ nsresult ChildDNSService::AsyncResolveInternal(
 }
 
 nsresult ChildDNSService::CancelAsyncResolveInternal(
-    const nsACString& aHostname, const nsACString& aTrrServer, uint16_t aType,
-    uint32_t aFlags, nsIDNSListener* aListener, nsresult aReason,
+    const nsACString& aHostname, uint16_t aType, uint32_t aFlags,
+    nsIDNSResolverInfo* aResolver, nsIDNSListener* aListener, nsresult aReason,
     const OriginAttributes& aOriginAttributes) {
   if (mDisablePrefetch && (aFlags & RESOLVE_SPECULATE)) {
     return NS_ERROR_DNS_LOOKUP_QUEUE_FULL;
@@ -171,8 +155,8 @@ nsresult ChildDNSService::CancelAsyncResolveInternal(
   nsTArray<RefPtr<DNSRequestSender>>* hashEntry;
   nsCString key;
   uintptr_t listenerAddr = reinterpret_cast<uintptr_t>(aListener);
-  GetDNSRecordHashKey(aHostname, aTrrServer, aType, aOriginAttributes, aFlags,
-                      listenerAddr, key);
+  GetDNSRecordHashKey(aHostname, DNSResolverInfo::URL(aResolver), aType,
+                      aOriginAttributes, aFlags, listenerAddr, key);
   if (mPendingRequests.Get(key, &hashEntry)) {
     // We cancel just one.
     hashEntry->ElementAt(0)->Cancel(aReason);
@@ -186,7 +170,9 @@ nsresult ChildDNSService::CancelAsyncResolveInternal(
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-ChildDNSService::AsyncResolve(const nsACString& hostname, uint32_t flags,
+ChildDNSService::AsyncResolve(const nsACString& hostname,
+                              nsIDNSService::ResolveType aType, uint32_t flags,
+                              nsIDNSResolverInfo* aResolver,
                               nsIDNSListener* listener, nsIEventTarget* target_,
                               JS::HandleValue aOriginAttributes, JSContext* aCx,
                               uint8_t aArgc, nsICancelable** result) {
@@ -198,83 +184,34 @@ ChildDNSService::AsyncResolve(const nsACString& hostname, uint32_t flags,
     }
   }
 
-  return AsyncResolveInternal(hostname, EmptyCString(),
-                              nsIDNSService::RESOLVE_TYPE_DEFAULT, flags,
-                              listener, target_, attrs, result);
-}
-
-NS_IMETHODIMP
-ChildDNSService::AsyncResolveNative(const nsACString& hostname, uint32_t flags,
-                                    nsIDNSListener* listener,
-                                    nsIEventTarget* target_,
-                                    const OriginAttributes& aOriginAttributes,
-                                    nsICancelable** result) {
-  return AsyncResolveInternal(hostname, EmptyCString(),
-                              nsIDNSService::RESOLVE_TYPE_DEFAULT, flags,
-                              listener, target_, aOriginAttributes, result);
-}
-
-NS_IMETHODIMP
-ChildDNSService::AsyncResolveWithTrrServer(
-    const nsACString& hostname, const nsACString& trrServer, uint32_t flags,
-    nsIDNSListener* listener, nsIEventTarget* target_,
-    JS::HandleValue aOriginAttributes, JSContext* aCx, uint8_t aArgc,
-    nsICancelable** result) {
-  OriginAttributes attrs;
-
-  if (aArgc == 1) {
-    if (!aOriginAttributes.isObject() || !attrs.Init(aCx, aOriginAttributes)) {
-      return NS_ERROR_INVALID_ARG;
-    }
-  }
-
-  return AsyncResolveInternal(hostname, trrServer,
-                              nsIDNSService::RESOLVE_TYPE_DEFAULT, flags,
-                              listener, target_, attrs, result);
-}
-
-NS_IMETHODIMP
-ChildDNSService::AsyncResolveWithTrrServerNative(
-    const nsACString& hostname, const nsACString& trrServer, uint32_t flags,
-    nsIDNSListener* listener, nsIEventTarget* target_,
-    const OriginAttributes& aOriginAttributes, nsICancelable** result) {
-  return AsyncResolveInternal(hostname, trrServer,
-                              nsIDNSService::RESOLVE_TYPE_DEFAULT, flags,
-                              listener, target_, aOriginAttributes, result);
-}
-
-NS_IMETHODIMP
-ChildDNSService::AsyncResolveByType(const nsACString& hostname, uint16_t type,
-                                    uint32_t flags, nsIDNSListener* listener,
-                                    nsIEventTarget* target_,
-                                    JS::HandleValue aOriginAttributes,
-                                    JSContext* aCx, uint8_t aArgc,
-                                    nsICancelable** result) {
-  OriginAttributes attrs;
-
-  if (aArgc == 1) {
-    if (!aOriginAttributes.isObject() || !attrs.Init(aCx, aOriginAttributes)) {
-      return NS_ERROR_INVALID_ARG;
-    }
-  }
-
-  return AsyncResolveInternal(hostname, EmptyCString(), type, flags, listener,
+  return AsyncResolveInternal(hostname, aType, flags, aResolver, listener,
                               target_, attrs, result);
 }
 
 NS_IMETHODIMP
-ChildDNSService::AsyncResolveByTypeNative(
-    const nsACString& hostname, uint16_t type, uint32_t flags,
-    nsIDNSListener* listener, nsIEventTarget* target_,
-    const OriginAttributes& aOriginAttributes, nsICancelable** result) {
-  return AsyncResolveInternal(hostname, EmptyCString(), type, flags, listener,
+ChildDNSService::AsyncResolveNative(
+    const nsACString& hostname, nsIDNSService::ResolveType aType,
+    uint32_t flags, nsIDNSResolverInfo* aResolver, nsIDNSListener* listener,
+    nsIEventTarget* target_, const OriginAttributes& aOriginAttributes,
+    nsICancelable** result) {
+  return AsyncResolveInternal(hostname, aType, flags, aResolver, listener,
                               target_, aOriginAttributes, result);
 }
 
+NS_IMETHODIMP
+ChildDNSService::NewTRRResolverInfo(const nsACString& aTrrURL,
+                                    nsIDNSResolverInfo** aResolver) {
+  RefPtr<DNSResolverInfo> res = new DNSResolverInfo(aTrrURL);
+  res.forget(aResolver);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 ChildDNSService::CancelAsyncResolve(const nsACString& aHostname,
-                                    uint32_t aFlags, nsIDNSListener* aListener,
-                                    nsresult aReason,
+                                    nsIDNSService::ResolveType aType,
+                                    uint32_t aFlags,
+                                    nsIDNSResolverInfo* aResolver,
+                                    nsIDNSListener* aListener, nsresult aReason,
                                     JS::HandleValue aOriginAttributes,
                                     JSContext* aCx, uint8_t aArgc) {
   OriginAttributes attrs;
@@ -285,73 +222,16 @@ ChildDNSService::CancelAsyncResolve(const nsACString& aHostname,
     }
   }
 
-  return CancelAsyncResolveInternal(aHostname, EmptyCString(),
-                                    nsIDNSService::RESOLVE_TYPE_DEFAULT, aFlags,
+  return CancelAsyncResolveInternal(aHostname, aType, aFlags, aResolver,
                                     aListener, aReason, attrs);
 }
 
 NS_IMETHODIMP
 ChildDNSService::CancelAsyncResolveNative(
-    const nsACString& aHostname, uint32_t aFlags, nsIDNSListener* aListener,
+    const nsACString& aHostname, nsIDNSService::ResolveType aType,
+    uint32_t aFlags, nsIDNSResolverInfo* aResolver, nsIDNSListener* aListener,
     nsresult aReason, const OriginAttributes& aOriginAttributes) {
-  return CancelAsyncResolveInternal(aHostname, EmptyCString(),
-                                    nsIDNSService::RESOLVE_TYPE_DEFAULT, aFlags,
-                                    aListener, aReason, aOriginAttributes);
-}
-
-NS_IMETHODIMP
-ChildDNSService::CancelAsyncResolveWithTrrServer(
-    const nsACString& aHostname, const nsACString& aTrrServer, uint32_t aFlags,
-    nsIDNSListener* aListener, nsresult aReason,
-    JS::HandleValue aOriginAttributes, JSContext* aCx, uint8_t aArgc) {
-  OriginAttributes attrs;
-
-  if (aArgc == 1) {
-    if (!aOriginAttributes.isObject() || !attrs.Init(aCx, aOriginAttributes)) {
-      return NS_ERROR_INVALID_ARG;
-    }
-  }
-
-  return CancelAsyncResolveInternal(aHostname, aTrrServer,
-                                    nsIDNSService::RESOLVE_TYPE_DEFAULT, aFlags,
-                                    aListener, aReason, attrs);
-}
-
-NS_IMETHODIMP
-ChildDNSService::CancelAsyncResolveWithTrrServerNative(
-    const nsACString& aHostname, const nsACString& aTrrServer, uint32_t aFlags,
-    nsIDNSListener* aListener, nsresult aReason,
-    const OriginAttributes& aOriginAttributes) {
-  return CancelAsyncResolveInternal(aHostname, aTrrServer,
-                                    nsIDNSService::RESOLVE_TYPE_DEFAULT, aFlags,
-                                    aListener, aReason, aOriginAttributes);
-}
-
-NS_IMETHODIMP
-ChildDNSService::CancelAsyncResolveByType(const nsACString& aHostname,
-                                          uint16_t aType, uint32_t aFlags,
-                                          nsIDNSListener* aListener,
-                                          nsresult aReason,
-                                          JS::HandleValue aOriginAttributes,
-                                          JSContext* aCx, uint8_t aArgc) {
-  OriginAttributes attrs;
-
-  if (aArgc == 1) {
-    if (!aOriginAttributes.isObject() || !attrs.Init(aCx, aOriginAttributes)) {
-      return NS_ERROR_INVALID_ARG;
-    }
-  }
-
-  return CancelAsyncResolveInternal(aHostname, EmptyCString(), aType, aFlags,
-                                    aListener, aReason, attrs);
-}
-
-NS_IMETHODIMP
-ChildDNSService::CancelAsyncResolveByTypeNative(
-    const nsACString& aHostname, uint16_t aType, uint32_t aFlags,
-    nsIDNSListener* aListener, nsresult aReason,
-    const OriginAttributes& aOriginAttributes) {
-  return CancelAsyncResolveInternal(aHostname, EmptyCString(), aType, aFlags,
+  return CancelAsyncResolveInternal(aHostname, aType, aFlags, aResolver,
                                     aListener, aReason, aOriginAttributes);
 }
 
@@ -421,7 +301,7 @@ ChildDNSService::GetCurrentTrrURI(nsACString& aURI) {
 }
 
 NS_IMETHODIMP
-ChildDNSService::GetCurrentTrrMode(uint32_t* aMode) {
+ChildDNSService::GetCurrentTrrMode(nsIDNSService::ResolverMode* aMode) {
   if (!mTRRServiceParent) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -430,12 +310,34 @@ ChildDNSService::GetCurrentTrrMode(uint32_t* aMode) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+ChildDNSService::GetCurrentTrrConfirmationState(uint32_t* aConfirmationState) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP
 ChildDNSService::GetMyHostName(nsACString& result) {
+  if (XRE_IsParentProcess()) {
+    char name[100];
+    if (PR_GetSystemInfo(PR_SI_HOSTNAME, name, sizeof(name)) == PR_SUCCESS) {
+      result = name;
+      return NS_OK;
+    }
+
+    return NS_ERROR_FAILURE;
+  }
   // TODO: get value from parent during PNecko construction?
   return NS_ERROR_NOT_AVAILABLE;
 }
 
+NS_IMETHODIMP
+ChildDNSService::GetODoHActivated(bool* aResult) {
+  NS_ENSURE_ARG(aResult);
+
+  *aResult = mODoHActivated;
+  return NS_OK;
+}
+
 void ChildDNSService::NotifyRequestDone(DNSRequestSender* aDnsRequest) {
   // We need the original flags and listener for the pending requests hash.
   uint32_t originalFlags = aDnsRequest->mFlags & ~RESOLVE_OFFLINE;
@@ -474,11 +376,9 @@ nsresult ChildDNSService::Init() {
   // Disable prefetching either by explicit preference or if a manual proxy
   // is configured
   bool disablePrefetch = false;
-  int proxyType = nsIProtocolProxyService::PROXYCONFIG_DIRECT;
 
   nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
   if (prefs) {
-    prefs->GetIntPref("network.proxy.type", &proxyType);
     prefs->GetBoolPref(kPrefNameDisablePrefetch, &disablePrefetch);
   }
 
@@ -491,10 +391,17 @@ nsresult ChildDNSService::Init() {
       // If a manual proxy is in use, disable prefetch implicitly
       prefs->AddObserver("network.proxy.type", this, false);
     }
+
+    nsCOMPtr<nsIObserverService> observerService =
+        mozilla::services::GetObserverService();
+    if (observerService) {
+      observerService->AddObserver(this, "odoh-service-activated", false);
+    }
   }
 
-  mDisablePrefetch = disablePrefetch ||
-                     (proxyType == nsIProtocolProxyService::PROXYCONFIG_MANUAL);
+  mDisablePrefetch =
+      disablePrefetch || (StaticPrefs::network_proxy_type() ==
+                          nsIProtocolProxyService::PROXYCONFIG_MANUAL);
 
   return NS_OK;
 }
@@ -513,6 +420,24 @@ ChildDNSService::SetPrefetchEnabled(bool inVal) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+ChildDNSService::ReportFailedSVCDomainName(const nsACString& aOwnerName,
+                                           const nsACString& aSVCDomainName) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+ChildDNSService::IsSVCDomainNameFailed(const nsACString& aOwnerName,
+                                       const nsACString& aSVCDomainName,
+                                       bool* aResult) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+ChildDNSService::ResetExcludedSVCDomainName(const nsACString& aOwnerName) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 //-----------------------------------------------------------------------------
 // ChildDNSService::nsIObserver
 //-----------------------------------------------------------------------------
@@ -520,12 +445,13 @@ ChildDNSService::SetPrefetchEnabled(bool inVal) {
 NS_IMETHODIMP
 ChildDNSService::Observe(nsISupports* subject, const char* topic,
                          const char16_t* data) {
-  // we are only getting called if a preference has changed.
-  NS_ASSERTION(strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID) == 0,
-               "unexpected observe call");
+  if (!strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
+    // Reread prefs
+    Init();
+  } else if (!strcmp(topic, "odoh-service-activated")) {
+    mODoHActivated = u"true"_ns.Equals(data);
+  }
 
-  // Reread prefs
-  Init();
   return NS_OK;
 }
 
