@@ -26,6 +26,7 @@ extern crate winapi;
 
 use pkcs11::types::*;
 use std::sync::Mutex;
+use std::thread;
 
 mod manager;
 #[macro_use]
@@ -35,7 +36,7 @@ mod backend_macos;
 #[cfg(target_os = "windows")]
 mod backend_windows;
 
-use manager::ManagerProxy;
+use manager::{ManagerProxy, SlotType};
 
 lazy_static! {
     /// The singleton `ManagerProxy` that handles state with respect to PKCS #11. Only one thread
@@ -58,7 +59,8 @@ macro_rules! try_to_get_manager_guard {
         match MANAGER_PROXY.lock() {
             Ok(maybe_manager_proxy) => maybe_manager_proxy,
             Err(poison_error) => {
-                error!(
+                log_with_thread_id!(
+                    error,
                     "previous thread panicked acquiring manager lock: {}",
                     poison_error
                 );
@@ -73,13 +75,21 @@ macro_rules! manager_guard_to_manager {
         match $manager_guard.as_mut() {
             Some(manager_proxy) => manager_proxy,
             None => {
-                error!("manager expected to be set, but it is not");
+                log_with_thread_id!(error, "manager expected to be set, but it is not");
                 return CKR_DEVICE_ERROR;
             }
         }
     };
 }
 
+// Helper macro to prefix log messages with the current thread ID.
+macro_rules! log_with_thread_id {
+    ($log_level:ident, $($message:expr),*) => {
+        let message = format!($($message),*);
+        $log_level!("{:?} {}", thread::current().id(), message);
+    };
+}
+
 /// This gets called to initialize the module. For this implementation, this consists of
 /// instantiating the `ManagerProxy`.
 extern "C" fn C_Initialize(_pInitArgs: CK_C_INITIALIZE_ARGS_PTR) -> CK_RV {
@@ -87,20 +97,24 @@ extern "C" fn C_Initialize(_pInitArgs: CK_C_INITIALIZE_ARGS_PTR) -> CK_RV {
     // logging has been initialized.
     let _ = env_logger::try_init();
     let mut manager_guard = try_to_get_manager_guard!();
-    match manager_guard.replace(ManagerProxy::new()) {
+    let manager_proxy = match ManagerProxy::new() {
+        Ok(p) => p,
+        Err(()) => return CKR_DEVICE_ERROR,
+    };
+    match manager_guard.replace(manager_proxy) {
         Some(_unexpected_previous_manager) => {
             #[cfg(target_os = "macos")]
             {
-                info!("C_Initialize: manager previously set (this is expected on macOS - replacing it)");
+                log_with_thread_id!(info, "C_Initialize: manager previously set (this is expected on macOS - replacing it)");
             }
             #[cfg(target_os = "windows")]
             {
-                warn!("C_Initialize: manager unexpectedly previously set (bravely continuing by replacing it)");
+                log_with_thread_id!(warn, "C_Initialize: manager unexpectedly previously set (bravely continuing by replacing it)");
             }
         }
         None => {}
     }
-    debug!("C_Initialize: CKR_OK");
+    log_with_thread_id!(debug, "C_Initialize: CKR_OK");
     CKR_OK
 }
 
@@ -109,11 +123,11 @@ extern "C" fn C_Finalize(_pReserved: CK_VOID_PTR) -> CK_RV {
     let manager = manager_guard_to_manager!(manager_guard);
     match manager.stop() {
         Ok(()) => {
-            debug!("C_Finalize: CKR_OK");
+            log_with_thread_id!(debug, "C_Finalize: CKR_OK");
             CKR_OK
         }
         Err(()) => {
-            debug!("C_Finalize: CKR_DEVICE_ERROR");
+            log_with_thread_id!(error, "C_Finalize: CKR_DEVICE_ERROR");
             CKR_DEVICE_ERROR
         }
     }
@@ -129,10 +143,10 @@ const LIBRARY_DESCRIPTION_BYTES: &[u8; 32] = b"OS Client Cert Module           "
 /// supports (portions of) cryptoki (PKCS #11) version 2.2.
 extern "C" fn C_GetInfo(pInfo: CK_INFO_PTR) -> CK_RV {
     if pInfo.is_null() {
-        error!("C_GetInfo: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_GetInfo: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
-    debug!("C_GetInfo: CKR_OK");
+    log_with_thread_id!(debug, "C_GetInfo: CKR_OK");
     let mut info = CK_INFO::default();
     info.cryptokiVersion.major = 2;
     info.cryptokiVersion.minor = 2;
@@ -144,8 +158,12 @@ extern "C" fn C_GetInfo(pInfo: CK_INFO_PTR) -> CK_RV {
     CKR_OK
 }
 
-/// This module only has one slot. Its ID is 1.
-const SLOT_ID: CK_SLOT_ID = 1;
+/// This module has two slots.
+const SLOT_COUNT: CK_ULONG = 2;
+/// The slot with ID 1 supports modern mechanisms like RSA-PSS.
+const SLOT_ID_MODERN: CK_SLOT_ID = 1;
+/// The slot with ID 2 only supports legacy mechanisms.
+const SLOT_ID_LEGACY: CK_SLOT_ID = 2;
 
 /// This gets called twice: once with a null `pSlotList` to get the number of slots (returned via
 /// `pulCount`) and a second time to get the ID for each slot.
@@ -155,38 +173,45 @@ extern "C" fn C_GetSlotList(
     pulCount: CK_ULONG_PTR,
 ) -> CK_RV {
     if pulCount.is_null() {
-        error!("C_GetSlotList: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_GetSlotList: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
-    unsafe {
-        *pulCount = 1;
-    }
     if !pSlotList.is_null() {
-        let slotCount = unsafe { *pulCount };
-        if slotCount < 1 {
-            error!("C_GetSlotList: CKR_BUFFER_TOO_SMALL");
+        if unsafe { *pulCount } < SLOT_COUNT {
+            log_with_thread_id!(error, "C_GetSlotList: CKR_BUFFER_TOO_SMALL");
             return CKR_BUFFER_TOO_SMALL;
         }
         unsafe {
-            *pSlotList = SLOT_ID;
+            *pSlotList = SLOT_ID_MODERN;
+            *pSlotList.offset(1) = SLOT_ID_LEGACY;
         }
     };
-    debug!("C_GetSlotList: CKR_OK");
+    unsafe {
+        *pulCount = SLOT_COUNT;
+    }
+    log_with_thread_id!(debug, "C_GetSlotList: CKR_OK");
     CKR_OK
 }
 
-const SLOT_DESCRIPTION_BYTES: &[u8; 64] =
-    b"OS Client Cert Slot                                             ";
+const SLOT_DESCRIPTION_MODERN_BYTES: &[u8; 64] =
+    b"OS Client Cert Slot (Modern)                                    ";
+const SLOT_DESCRIPTION_LEGACY_BYTES: &[u8; 64] =
+    b"OS Client Cert Slot (Legacy)                                    ";
 
-/// This gets called to obtain information about slots. In this implementation, the token is always
-/// present in the slot.
+/// This gets called to obtain information about slots. In this implementation, the tokens are
+/// always present in the slots.
 extern "C" fn C_GetSlotInfo(slotID: CK_SLOT_ID, pInfo: CK_SLOT_INFO_PTR) -> CK_RV {
-    if slotID != SLOT_ID || pInfo.is_null() {
-        error!("C_GetSlotInfo: CKR_ARGUMENTS_BAD");
+    if (slotID != SLOT_ID_MODERN && slotID != SLOT_ID_LEGACY) || pInfo.is_null() {
+        log_with_thread_id!(error, "C_GetSlotInfo: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
+    let description = if slotID == SLOT_ID_MODERN {
+        SLOT_DESCRIPTION_MODERN_BYTES
+    } else {
+        SLOT_DESCRIPTION_LEGACY_BYTES
+    };
     let slot_info = CK_SLOT_INFO {
-        slotDescription: *SLOT_DESCRIPTION_BYTES,
+        slotDescription: *description,
         manufacturerID: *MANUFACTURER_ID_BYTES,
         flags: CKF_TOKEN_PRESENT,
         hardwareVersion: CK_VERSION::default(),
@@ -195,48 +220,58 @@ extern "C" fn C_GetSlotInfo(slotID: CK_SLOT_ID, pInfo: CK_SLOT_INFO_PTR) -> CK_R
     unsafe {
         *pInfo = slot_info;
     }
-    debug!("C_GetSlotInfo: CKR_OK");
+    log_with_thread_id!(debug, "C_GetSlotInfo: CKR_OK");
     CKR_OK
 }
 
-const TOKEN_LABEL_BYTES: &[u8; 32] = b"OS Client Cert Token            ";
+const TOKEN_LABEL_MODERN_BYTES: &[u8; 32] = b"OS Client Cert Token (Modern)   ";
+const TOKEN_LABEL_LEGACY_BYTES: &[u8; 32] = b"OS Client Cert Token (Legacy)   ";
 const TOKEN_MODEL_BYTES: &[u8; 16] = b"osclientcerts   ";
 const TOKEN_SERIAL_NUMBER_BYTES: &[u8; 16] = b"0000000000000000";
 
-/// This gets called to obtain some information about tokens. This implementation only has one slot,
-/// so it only has one token. This information is primarily for display purposes.
+/// This gets called to obtain some information about tokens. This implementation has two slots,
+/// so it has two tokens. This information is primarily for display purposes.
 extern "C" fn C_GetTokenInfo(slotID: CK_SLOT_ID, pInfo: CK_TOKEN_INFO_PTR) -> CK_RV {
-    if slotID != SLOT_ID || pInfo.is_null() {
-        error!("C_GetTokenInfo: CKR_ARGUMENTS_BAD");
+    if (slotID != SLOT_ID_MODERN && slotID != SLOT_ID_LEGACY) || pInfo.is_null() {
+        log_with_thread_id!(error, "C_GetTokenInfo: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut token_info = CK_TOKEN_INFO::default();
-    token_info.label = *TOKEN_LABEL_BYTES;
+    let label = if slotID == SLOT_ID_MODERN {
+        TOKEN_LABEL_MODERN_BYTES
+    } else {
+        TOKEN_LABEL_LEGACY_BYTES
+    };
+    token_info.label = *label;
     token_info.manufacturerID = *MANUFACTURER_ID_BYTES;
     token_info.model = *TOKEN_MODEL_BYTES;
     token_info.serialNumber = *TOKEN_SERIAL_NUMBER_BYTES;
     unsafe {
         *pInfo = token_info;
     }
-    debug!("C_GetTokenInfo: CKR_OK");
+    log_with_thread_id!(debug, "C_GetTokenInfo: CKR_OK");
     CKR_OK
 }
 
-/// This gets called to determine what mechanisms a slot supports. This implementation supports
-/// ECDSA, RSA PKCS, and RSA PSS.
+/// This gets called to determine what mechanisms a slot supports. The modern slot supports ECDSA,
+/// RSA PKCS, and RSA PSS. The legacy slot only supports RSA PKCS.
 extern "C" fn C_GetMechanismList(
     slotID: CK_SLOT_ID,
     pMechanismList: CK_MECHANISM_TYPE_PTR,
     pulCount: CK_ULONG_PTR,
 ) -> CK_RV {
-    if slotID != SLOT_ID || pulCount.is_null() {
-        error!("C_GetMechanismList: CKR_ARGUMENTS_BAD");
+    if (slotID != SLOT_ID_MODERN && slotID != SLOT_ID_LEGACY) || pulCount.is_null() {
+        log_with_thread_id!(error, "C_GetMechanismList: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
-    let mechanisms = [CKM_ECDSA, CKM_RSA_PKCS, CKM_RSA_PKCS_PSS];
+    let mechanisms = if slotID == SLOT_ID_MODERN {
+        vec![CKM_ECDSA, CKM_RSA_PKCS, CKM_RSA_PKCS_PSS]
+    } else {
+        vec![CKM_RSA_PKCS]
+    };
     if !pMechanismList.is_null() {
         if unsafe { *pulCount as usize } < mechanisms.len() {
-            error!("C_GetMechanismList: CKR_ARGUMENTS_BAD");
+            log_with_thread_id!(error, "C_GetMechanismList: CKR_ARGUMENTS_BAD");
             return CKR_ARGUMENTS_BAD;
         }
         for i in 0..mechanisms.len() {
@@ -248,7 +283,7 @@ extern "C" fn C_GetMechanismList(
     unsafe {
         *pulCount = mechanisms.len() as CK_ULONG;
     }
-    debug!("C_GetMechanismList: CKR_OK");
+    log_with_thread_id!(debug, "C_GetMechanismList: CKR_OK");
     CKR_OK
 }
 
@@ -257,7 +292,7 @@ extern "C" fn C_GetMechanismInfo(
     _type: CK_MECHANISM_TYPE,
     _pInfo: CK_MECHANISM_INFO_PTR,
 ) -> CK_RV {
-    error!("C_GetMechanismInfo: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GetMechanismInfo: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -267,7 +302,7 @@ extern "C" fn C_InitToken(
     _ulPinLen: CK_ULONG,
     _pLabel: CK_UTF8CHAR_PTR,
 ) -> CK_RV {
-    error!("C_InitToken: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_InitToken: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -276,7 +311,7 @@ extern "C" fn C_InitPIN(
     _pPin: CK_UTF8CHAR_PTR,
     _ulPinLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_InitPIN: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_InitPIN: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -287,7 +322,7 @@ extern "C" fn C_SetPIN(
     _pNewPin: CK_UTF8CHAR_PTR,
     _ulNewLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_SetPIN: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SetPIN: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -300,23 +335,28 @@ extern "C" fn C_OpenSession(
     _Notify: CK_NOTIFY,
     phSession: CK_SESSION_HANDLE_PTR,
 ) -> CK_RV {
-    if slotID != SLOT_ID || phSession.is_null() {
-        error!("C_OpenSession: CKR_ARGUMENTS_BAD");
+    if (slotID != SLOT_ID_MODERN && slotID != SLOT_ID_LEGACY) || phSession.is_null() {
+        log_with_thread_id!(error, "C_OpenSession: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut manager_guard = try_to_get_manager_guard!();
     let manager = manager_guard_to_manager!(manager_guard);
-    let session_handle = match manager.open_session() {
+    let slot_type = if slotID == SLOT_ID_MODERN {
+        SlotType::Modern
+    } else {
+        SlotType::Legacy
+    };
+    let session_handle = match manager.open_session(slot_type) {
         Ok(session_handle) => session_handle,
         Err(()) => {
-            error!("C_OpenSession: open_session failed");
+            log_with_thread_id!(error, "C_OpenSession: open_session failed");
             return CKR_DEVICE_ERROR;
         }
     };
     unsafe {
         *phSession = session_handle;
     }
-    debug!("C_OpenSession: CKR_OK");
+    log_with_thread_id!(debug, "C_OpenSession: CKR_OK");
     CKR_OK
 }
 
@@ -325,35 +365,40 @@ extern "C" fn C_CloseSession(hSession: CK_SESSION_HANDLE) -> CK_RV {
     let mut manager_guard = try_to_get_manager_guard!();
     let manager = manager_guard_to_manager!(manager_guard);
     if manager.close_session(hSession).is_err() {
-        error!("C_CloseSession: CKR_SESSION_HANDLE_INVALID");
+        log_with_thread_id!(error, "C_CloseSession: CKR_SESSION_HANDLE_INVALID");
         return CKR_SESSION_HANDLE_INVALID;
     }
-    debug!("C_CloseSession: CKR_OK");
+    log_with_thread_id!(debug, "C_CloseSession: CKR_OK");
     CKR_OK
 }
 
 /// This gets called to close all open sessions at once. This is handled by the `ManagerProxy`.
 extern "C" fn C_CloseAllSessions(slotID: CK_SLOT_ID) -> CK_RV {
-    if slotID != SLOT_ID {
-        error!("C_CloseAllSessions: CKR_ARGUMENTS_BAD");
+    if slotID != SLOT_ID_MODERN && slotID != SLOT_ID_LEGACY {
+        log_with_thread_id!(error, "C_CloseAllSessions: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut manager_guard = try_to_get_manager_guard!();
     let manager = manager_guard_to_manager!(manager_guard);
-    match manager.close_all_sessions() {
+    let slot_type = if slotID == SLOT_ID_MODERN {
+        SlotType::Modern
+    } else {
+        SlotType::Legacy
+    };
+    match manager.close_all_sessions(slot_type) {
         Ok(()) => {
-            debug!("C_CloseAllSessions: CKR_OK");
+            log_with_thread_id!(debug, "C_CloseAllSessions: CKR_OK");
             CKR_OK
         }
         Err(()) => {
-            debug!("C_CloseAllSessions: close_all_sessions failed");
+            log_with_thread_id!(error, "C_CloseAllSessions: close_all_sessions failed");
             CKR_DEVICE_ERROR
         }
     }
 }
 
 extern "C" fn C_GetSessionInfo(_hSession: CK_SESSION_HANDLE, _pInfo: CK_SESSION_INFO_PTR) -> CK_RV {
-    error!("C_GetSessionInfo: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GetSessionInfo: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -362,7 +407,7 @@ extern "C" fn C_GetOperationState(
     _pOperationState: CK_BYTE_PTR,
     _pulOperationStateLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_GetOperationState: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GetOperationState: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -373,7 +418,7 @@ extern "C" fn C_SetOperationState(
     _hEncryptionKey: CK_OBJECT_HANDLE,
     _hAuthenticationKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_SetOperationState: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SetOperationState: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -383,7 +428,7 @@ extern "C" fn C_Login(
     _pPin: CK_UTF8CHAR_PTR,
     _ulPinLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_Login: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_Login: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -391,7 +436,7 @@ extern "C" fn C_Login(
 /// hold on to authenticated resources, this module "implements" this by doing nothing and
 /// returning a success result.
 extern "C" fn C_Logout(_hSession: CK_SESSION_HANDLE) -> CK_RV {
-    debug!("C_Logout: CKR_OK");
+    log_with_thread_id!(debug, "C_Logout: CKR_OK");
     CKR_OK
 }
 
@@ -401,7 +446,7 @@ extern "C" fn C_CreateObject(
     _ulCount: CK_ULONG,
     _phObject: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_CreateObject: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_CreateObject: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -412,12 +457,12 @@ extern "C" fn C_CopyObject(
     _ulCount: CK_ULONG,
     _phNewObject: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_CopyObject: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_CopyObject: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
 extern "C" fn C_DestroyObject(_hSession: CK_SESSION_HANDLE, _hObject: CK_OBJECT_HANDLE) -> CK_RV {
-    error!("C_DestroyObject: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DestroyObject: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -426,7 +471,7 @@ extern "C" fn C_GetObjectSize(
     _hObject: CK_OBJECT_HANDLE,
     _pulSize: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_GetObjectSize: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GetObjectSize: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -443,7 +488,7 @@ extern "C" fn C_GetAttributeValue(
     ulCount: CK_ULONG,
 ) -> CK_RV {
     if pTemplate.is_null() {
-        error!("C_GetAttributeValue: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_GetAttributeValue: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut attr_types = Vec::with_capacity(ulCount as usize);
@@ -456,12 +501,13 @@ extern "C" fn C_GetAttributeValue(
     let values = match manager.get_attributes(hObject, attr_types) {
         Ok(values) => values,
         Err(()) => {
-            error!("C_GetAttributeValue: CKR_ARGUMENTS_BAD");
+            log_with_thread_id!(error, "C_GetAttributeValue: CKR_ARGUMENTS_BAD");
             return CKR_ARGUMENTS_BAD;
         }
     };
     if values.len() != ulCount as usize {
-        error!(
+        log_with_thread_id!(
+            error,
             "C_GetAttributeValue: manager.get_attributes didn't return the right number of values"
         );
         return CKR_DEVICE_ERROR;
@@ -475,7 +521,7 @@ extern "C" fn C_GetAttributeValue(
             } else {
                 let ptr: *mut u8 = attr.pValue as *mut u8;
                 if attr_value.len() != attr.ulValueLen as usize {
-                    error!("C_GetAttributeValue: incorrect attr size");
+                    log_with_thread_id!(error, "C_GetAttributeValue: incorrect attr size");
                     return CKR_ARGUMENTS_BAD;
                 }
                 unsafe {
@@ -486,7 +532,7 @@ extern "C" fn C_GetAttributeValue(
             attr.ulValueLen = (0 - 1) as CK_ULONG;
         }
     }
-    debug!("C_GetAttributeValue: CKR_OK");
+    log_with_thread_id!(debug, "C_GetAttributeValue: CKR_OK");
     CKR_OK
 }
 
@@ -496,10 +542,38 @@ extern "C" fn C_SetAttributeValue(
     _pTemplate: CK_ATTRIBUTE_PTR,
     _ulCount: CK_ULONG,
 ) -> CK_RV {
-    error!("C_SetAttributeValue: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SetAttributeValue: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
+fn trace_attr(prefix: &str, attr: &CK_ATTRIBUTE) {
+    let typ = match unsafe_packed_field_access!(attr.attrType) {
+        CKA_CLASS => "CKA_CLASS".to_string(),
+        CKA_TOKEN => "CKA_TOKEN".to_string(),
+        CKA_LABEL => "CKA_LABEL".to_string(),
+        CKA_ID => "CKA_ID".to_string(),
+        CKA_VALUE => "CKA_VALUE".to_string(),
+        CKA_ISSUER => "CKA_ISSUER".to_string(),
+        CKA_SERIAL_NUMBER => "CKA_SERIAL_NUMBER".to_string(),
+        CKA_SUBJECT => "CKA_SUBJECT".to_string(),
+        CKA_PRIVATE => "CKA_PRIVATE".to_string(),
+        CKA_KEY_TYPE => "CKA_KEY_TYPE".to_string(),
+        CKA_MODULUS => "CKA_MODULUS".to_string(),
+        CKA_EC_PARAMS => "CKA_EC_PARAMS".to_string(),
+        _ => format!("0x{:x}", unsafe_packed_field_access!(attr.attrType)),
+    };
+    let value =
+        unsafe { std::slice::from_raw_parts(attr.pValue as *const u8, attr.ulValueLen as usize) };
+    log_with_thread_id!(
+        trace,
+        "{}CK_ATTRIBUTE {{ attrType: {}, pValue: {:?}, ulValueLen: {} }}",
+        prefix,
+        typ,
+        value,
+        unsafe_packed_field_access!(attr.ulValueLen)
+    );
+}
+
 /// This gets called to initialize a search for objects matching a given list of attributes. This
 /// module implements this by gathering the attributes and passing them to the `ManagerProxy` to
 /// start the search.
@@ -509,14 +583,14 @@ extern "C" fn C_FindObjectsInit(
     ulCount: CK_ULONG,
 ) -> CK_RV {
     if pTemplate.is_null() {
-        error!("C_FindObjectsInit: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_FindObjectsInit: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut attrs = Vec::new();
-    trace!("C_FindObjectsInit:");
+    log_with_thread_id!(trace, "C_FindObjectsInit:");
     for i in 0..ulCount {
         let attr = unsafe { &*pTemplate.offset(i as isize) };
-        trace!("  {:?}", attr);
+        trace_attr("  ", &attr);
         let slice = unsafe {
             std::slice::from_raw_parts(attr.pValue as *const u8, attr.ulValueLen as usize)
         };
@@ -527,11 +601,11 @@ extern "C" fn C_FindObjectsInit(
     match manager.start_search(hSession, attrs) {
         Ok(()) => {}
         Err(()) => {
-            error!("C_FindObjectsInit: CKR_ARGUMENTS_BAD");
+            log_with_thread_id!(error, "C_FindObjectsInit: CKR_ARGUMENTS_BAD");
             return CKR_ARGUMENTS_BAD;
         }
     }
-    debug!("C_FindObjectsInit: CKR_OK");
+    log_with_thread_id!(debug, "C_FindObjectsInit: CKR_OK");
     CKR_OK
 }
 
@@ -545,7 +619,7 @@ extern "C" fn C_FindObjects(
     pulObjectCount: CK_ULONG_PTR,
 ) -> CK_RV {
     if phObject.is_null() || pulObjectCount.is_null() || ulMaxObjectCount == 0 {
-        error!("C_FindObjects: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_FindObjects: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let mut manager_guard = try_to_get_manager_guard!();
@@ -553,13 +627,13 @@ extern "C" fn C_FindObjects(
     let handles = match manager.search(hSession, ulMaxObjectCount as usize) {
         Ok(handles) => handles,
         Err(()) => {
-            error!("C_FindObjects: CKR_ARGUMENTS_BAD");
+            log_with_thread_id!(error, "C_FindObjects: CKR_ARGUMENTS_BAD");
             return CKR_ARGUMENTS_BAD;
         }
     };
-    debug!("C_FindObjects: found handles {:?}", handles);
+    log_with_thread_id!(debug, "C_FindObjects: found handles {:?}", handles);
     if handles.len() > ulMaxObjectCount as usize {
-        error!("C_FindObjects: manager returned too many handles");
+        log_with_thread_id!(error, "C_FindObjects: manager returned too many handles");
         return CKR_DEVICE_ERROR;
     }
     unsafe {
@@ -572,7 +646,7 @@ extern "C" fn C_FindObjects(
             }
         }
     }
-    debug!("C_FindObjects: CKR_OK");
+    log_with_thread_id!(debug, "C_FindObjects: CKR_OK");
     CKR_OK
 }
 
@@ -584,11 +658,11 @@ extern "C" fn C_FindObjectsFinal(hSession: CK_SESSION_HANDLE) -> CK_RV {
     // It would be an error if there were no search for this session, but we can be permissive here.
     match manager.clear_search(hSession) {
         Ok(()) => {
-            debug!("C_FindObjectsFinal: CKR_OK");
+            log_with_thread_id!(debug, "C_FindObjectsFinal: CKR_OK");
             CKR_OK
         }
         Err(()) => {
-            debug!("C_FindObjectsFinal: clear_search failed");
+            log_with_thread_id!(error, "C_FindObjectsFinal: clear_search failed");
             CKR_DEVICE_ERROR
         }
     }
@@ -599,7 +673,7 @@ extern "C" fn C_EncryptInit(
     _pMechanism: CK_MECHANISM_PTR,
     _hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_EncryptInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_EncryptInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -610,7 +684,7 @@ extern "C" fn C_Encrypt(
     _pEncryptedData: CK_BYTE_PTR,
     _pulEncryptedDataLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_Encrypt: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_Encrypt: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -621,7 +695,7 @@ extern "C" fn C_EncryptUpdate(
     _pEncryptedPart: CK_BYTE_PTR,
     _pulEncryptedPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_EncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_EncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -630,7 +704,7 @@ extern "C" fn C_EncryptFinal(
     _pLastEncryptedPart: CK_BYTE_PTR,
     _pulLastEncryptedPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_EncryptFinal: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_EncryptFinal: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -639,7 +713,7 @@ extern "C" fn C_DecryptInit(
     _pMechanism: CK_MECHANISM_PTR,
     _hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_DecryptInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DecryptInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -650,7 +724,7 @@ extern "C" fn C_Decrypt(
     _pData: CK_BYTE_PTR,
     _pulDataLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_Decrypt: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_Decrypt: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -661,7 +735,7 @@ extern "C" fn C_DecryptUpdate(
     _pPart: CK_BYTE_PTR,
     _pulPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DecryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DecryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -670,12 +744,12 @@ extern "C" fn C_DecryptFinal(
     _pLastPart: CK_BYTE_PTR,
     _pulLastPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DecryptFinal: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DecryptFinal: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
 extern "C" fn C_DigestInit(_hSession: CK_SESSION_HANDLE, _pMechanism: CK_MECHANISM_PTR) -> CK_RV {
-    error!("C_DigestInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DigestInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -686,7 +760,7 @@ extern "C" fn C_Digest(
     _pDigest: CK_BYTE_PTR,
     _pulDigestLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_Digest: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_Digest: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -695,12 +769,12 @@ extern "C" fn C_DigestUpdate(
     _pPart: CK_BYTE_PTR,
     _ulPartLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_DigestUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DigestUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
 extern "C" fn C_DigestKey(_hSession: CK_SESSION_HANDLE, _hKey: CK_OBJECT_HANDLE) -> CK_RV {
-    error!("C_DigestKey: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DigestKey: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -709,7 +783,7 @@ extern "C" fn C_DigestFinal(
     _pDigest: CK_BYTE_PTR,
     _pulDigestLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DigestFinal: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DigestFinal: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -721,16 +795,17 @@ extern "C" fn C_SignInit(
     hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
     if pMechanism.is_null() {
-        error!("C_SignInit: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_SignInit: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     // Presumably we should validate the mechanism against hKey, but the specification doesn't
     // actually seem to require this.
     let mechanism = unsafe { *pMechanism };
-    debug!("C_SignInit: mechanism is {:?}", mechanism);
+    log_with_thread_id!(debug, "C_SignInit: mechanism is {:?}", mechanism);
     let mechanism_params = if mechanism.mechanism == CKM_RSA_PKCS_PSS {
         if mechanism.ulParameterLen as usize != std::mem::size_of::<CK_RSA_PKCS_PSS_PARAMS>() {
-            error!(
+            log_with_thread_id!(
+                error,
                 "C_SignInit: bad ulParameterLen for CKM_RSA_PKCS_PSS: {}",
                 unsafe_packed_field_access!(mechanism.ulParameterLen)
             );
@@ -745,11 +820,11 @@ extern "C" fn C_SignInit(
     match manager.start_sign(hSession, hKey, mechanism_params) {
         Ok(()) => {}
         Err(()) => {
-            error!("C_SignInit: CKR_GENERAL_ERROR");
+            log_with_thread_id!(error, "C_SignInit: CKR_GENERAL_ERROR");
             return CKR_GENERAL_ERROR;
         }
     };
-    debug!("C_SignInit: CKR_OK");
+    log_with_thread_id!(debug, "C_SignInit: CKR_OK");
     CKR_OK
 }
 
@@ -764,7 +839,7 @@ extern "C" fn C_Sign(
     pulSignatureLen: CK_ULONG_PTR,
 ) -> CK_RV {
     if pData.is_null() || pulSignatureLen.is_null() {
-        error!("C_Sign: CKR_ARGUMENTS_BAD");
+        log_with_thread_id!(error, "C_Sign: CKR_ARGUMENTS_BAD");
         return CKR_ARGUMENTS_BAD;
     }
     let data = unsafe { std::slice::from_raw_parts(pData, ulDataLen as usize) };
@@ -776,7 +851,7 @@ extern "C" fn C_Sign(
                 *pulSignatureLen = signature_length as CK_ULONG;
             },
             Err(()) => {
-                error!("C_Sign: get_signature_length failed");
+                log_with_thread_id!(error, "C_Sign: get_signature_length failed");
                 return CKR_GENERAL_ERROR;
             }
         }
@@ -787,7 +862,7 @@ extern "C" fn C_Sign(
             Ok(signature) => {
                 let signature_capacity = unsafe { *pulSignatureLen } as usize;
                 if signature_capacity < signature.len() {
-                    error!("C_Sign: CKR_ARGUMENTS_BAD");
+                    log_with_thread_id!(error, "C_Sign: CKR_ARGUMENTS_BAD");
                     return CKR_ARGUMENTS_BAD;
                 }
                 let ptr: *mut u8 = pSignature as *mut u8;
@@ -797,12 +872,12 @@ extern "C" fn C_Sign(
                 }
             }
             Err(()) => {
-                error!("C_Sign: sign failed");
+                log_with_thread_id!(error, "C_Sign: sign failed");
                 return CKR_GENERAL_ERROR;
             }
         }
     }
-    debug!("C_Sign: CKR_OK");
+    log_with_thread_id!(debug, "C_Sign: CKR_OK");
     CKR_OK
 }
 
@@ -811,7 +886,7 @@ extern "C" fn C_SignUpdate(
     _pPart: CK_BYTE_PTR,
     _ulPartLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_SignUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SignUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -820,7 +895,7 @@ extern "C" fn C_SignFinal(
     _pSignature: CK_BYTE_PTR,
     _pulSignatureLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_SignFinal: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SignFinal: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -829,7 +904,7 @@ extern "C" fn C_SignRecoverInit(
     _pMechanism: CK_MECHANISM_PTR,
     _hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_SignRecoverInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SignRecoverInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -840,7 +915,7 @@ extern "C" fn C_SignRecover(
     _pSignature: CK_BYTE_PTR,
     _pulSignatureLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_SignRecover: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SignRecover: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -849,7 +924,7 @@ extern "C" fn C_VerifyInit(
     _pMechanism: CK_MECHANISM_PTR,
     _hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_VerifyInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_VerifyInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -860,7 +935,7 @@ extern "C" fn C_Verify(
     _pSignature: CK_BYTE_PTR,
     _ulSignatureLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_Verify: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_Verify: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -869,7 +944,7 @@ extern "C" fn C_VerifyUpdate(
     _pPart: CK_BYTE_PTR,
     _ulPartLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_VerifyUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_VerifyUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -878,7 +953,7 @@ extern "C" fn C_VerifyFinal(
     _pSignature: CK_BYTE_PTR,
     _ulSignatureLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_VerifyFinal: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_VerifyFinal: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -887,7 +962,7 @@ extern "C" fn C_VerifyRecoverInit(
     _pMechanism: CK_MECHANISM_PTR,
     _hKey: CK_OBJECT_HANDLE,
 ) -> CK_RV {
-    error!("C_VerifyRecoverInit: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_VerifyRecoverInit: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -898,7 +973,7 @@ extern "C" fn C_VerifyRecover(
     _pData: CK_BYTE_PTR,
     _pulDataLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_VerifyRecover: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_VerifyRecover: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -909,7 +984,7 @@ extern "C" fn C_DigestEncryptUpdate(
     _pEncryptedPart: CK_BYTE_PTR,
     _pulEncryptedPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DigestEncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DigestEncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -920,7 +995,7 @@ extern "C" fn C_DecryptDigestUpdate(
     _pPart: CK_BYTE_PTR,
     _pulPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DecryptDigestUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DecryptDigestUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -931,7 +1006,7 @@ extern "C" fn C_SignEncryptUpdate(
     _pEncryptedPart: CK_BYTE_PTR,
     _pulEncryptedPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_SignEncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SignEncryptUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -942,7 +1017,7 @@ extern "C" fn C_DecryptVerifyUpdate(
     _pPart: CK_BYTE_PTR,
     _pulPartLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_DecryptVerifyUpdate: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DecryptVerifyUpdate: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -953,7 +1028,7 @@ extern "C" fn C_GenerateKey(
     _ulCount: CK_ULONG,
     _phKey: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_GenerateKey: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GenerateKey: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -967,7 +1042,7 @@ extern "C" fn C_GenerateKeyPair(
     _phPublicKey: CK_OBJECT_HANDLE_PTR,
     _phPrivateKey: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_GenerateKeyPair: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GenerateKeyPair: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -979,7 +1054,7 @@ extern "C" fn C_WrapKey(
     _pWrappedKey: CK_BYTE_PTR,
     _pulWrappedKeyLen: CK_ULONG_PTR,
 ) -> CK_RV {
-    error!("C_WrapKey: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_WrapKey: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -993,7 +1068,7 @@ extern "C" fn C_UnwrapKey(
     _ulAttributeCount: CK_ULONG,
     _phKey: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_UnwrapKey: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_UnwrapKey: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -1005,7 +1080,7 @@ extern "C" fn C_DeriveKey(
     _ulAttributeCount: CK_ULONG,
     _phKey: CK_OBJECT_HANDLE_PTR,
 ) -> CK_RV {
-    error!("C_DeriveKey: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_DeriveKey: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -1014,7 +1089,7 @@ extern "C" fn C_SeedRandom(
     _pSeed: CK_BYTE_PTR,
     _ulSeedLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_SeedRandom: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_SeedRandom: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -1023,17 +1098,17 @@ extern "C" fn C_GenerateRandom(
     _RandomData: CK_BYTE_PTR,
     _ulRandomLen: CK_ULONG,
 ) -> CK_RV {
-    error!("C_GenerateRandom: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GenerateRandom: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
 extern "C" fn C_GetFunctionStatus(_hSession: CK_SESSION_HANDLE) -> CK_RV {
-    error!("C_GetFunctionStatus: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_GetFunctionStatus: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
 extern "C" fn C_CancelFunction(_hSession: CK_SESSION_HANDLE) -> CK_RV {
-    error!("C_CancelFunction: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_CancelFunction: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
@@ -1042,7 +1117,7 @@ extern "C" fn C_WaitForSlotEvent(
     _pSlot: CK_SLOT_ID_PTR,
     _pRserved: CK_VOID_PTR,
 ) -> CK_RV {
-    error!("C_WaitForSlotEvent: CKR_FUNCTION_NOT_SUPPORTED");
+    log_with_thread_id!(error, "C_WaitForSlotEvent: CKR_FUNCTION_NOT_SUPPORTED");
     CKR_FUNCTION_NOT_SUPPORTED
 }
 
